<?php

declare(strict_types=1);

namespace Drupal\graphql_nuxt_multi_cache\EventSubscriber;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\nuxt_multi_cache\CacheTagHelper;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Adds cache tag headers for the nuxt-multi-cache module.
 */
class AddCacheTags implements EventSubscriberInterface {

  /**
   * Constructs a new CacheTagsHeaderLimitDetector object.
   *
   * @param \Drupal\nuxt_multi_cache\CacheTagHelper $cacheTagHelper
   *   The cache tag helper.
   */
  public function __construct(
    protected CacheTagHelper $cacheTagHelper
  ) {
  }

  /**
   * Adds Nuxt Cache header.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The event to process.
   */
  public function onRespond(ResponseEvent $event): void {
    // Only modify the main request.
    if ((!$event->isMainRequest())) {
      return;
    }

    // Get response.
    // Drupal\Core\Cache\CacheableJsonResponse.
    $response = $event->getResponse();
    if ($response instanceof CacheableJsonResponse) {
      $metadata = $response->getCacheableMetadata();
      $cacheTags = $metadata->getCacheTags();
      $maxAge = $metadata->getCacheMaxAge();
      $filtered = $this->cacheTagHelper->filterBannedTags($cacheTags);
      $encoded = $this->cacheTagHelper->encodeTags($filtered);

      // Set the encoded and filtered cache tag header.
      $response->headers->set('X-Nuxt-Cache-Tags', implode(' ', $encoded));

      // Set an expire header.
      $time = $event->getRequest()->server->get('REQUEST_TIME');
      $expires = ($maxAge === Cache::PERMANENT) ? Cache::PERMANENT : (int) $time + $maxAge;
      $response->headers->set('X-Nuxt-Expires', (string) $expires);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::RESPONSE][] = ['onRespond'];
    return $events;
  }

}
