<?php

declare(strict_types=1);

namespace Drupal\nuxt_multi_cache;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;

/**
 * Cache tags helper.
 */
class CacheTagHelper {

  /**
   * Array of cache tag prefix to its short version.
   */
  protected array $prefixMap;

  /**
   * Array of banned tags.
   */
  protected array $banned;

  /**
   * The module config.
   */
  protected ImmutableConfig $config;

  /**
   * NuxtMultiCacheApi constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->config = $config_factory->get('nuxt_multi_cache.settings');
    $this->prefixMap = [
      'node' => 'N',
      'taxonomy_term' => 'T',
      'block_content' => 'B',
      'file' => 'F',
      'media' => 'M',
      'user' => 'U',
      'menu_link_content' => 'L',
      'commerce_product' => 'P',
      'commerce_product_variation' => 'V',
      'texts_context' => 'C',
      'translatable_config_pages' => 't',
      'rokka_metadata' => 'r',
    ];
    $this->banned = $this->config->get('ban_tags') ?? [];
  }

  /**
   * Filters an array of tags to remove banned tags.
   *
   * @param string[] $tags
   *   Tags to filter.
   *
   * @return string[]
   *   The filtered tags.
   */
  public function filterBannedTags(array $tags): array {
    return array_filter($tags, function ($tag) {
      return !in_array($tag, $this->banned);
    });
  }

  /**
   * Hash the cache tag.
   *
   * @param string $tag
   *   The tag to hash.
   *
   * @return string
   *   The hashed cache tag.
   */
  public function hashTag(string $tag): string {
    return substr(base_convert(md5($tag), 16, 36), 0, 4);
  }

  /**
   * Encode tags to their short form.
   */
  public function encodeTags(array $tags): array {
    $encoded = [];

    foreach ($tags as $tag) {
      $encoded[] = $this->encodeTag($tag);
    }

    return $encoded;
  }

  /**
   * Encode a tag.
   *
   * Tags in the form of entity_type:123 are encoded in a readable format, if
   * the entity type is mapped. For example, `node:123` becomes `N:123`. This
   * helps debugging caching issues by at least being able to see some common
   * tags.
   *
   * If no mapping is available, the tag is hashed to a 4 character string.
   *
   * @param string $tag
   *   The tag to encode.
   *
   * @return string
   *   The encoded tag.
   */
  public function encodeTag(string $tag): string {
    // Handle case of e.g. node:123.
    // ['node', '123']
    $parts = explode(':', $tag, 2);
    if (!empty($parts[1])) {
      $first = $parts[0];

      // Try to use a short form of the first part.
      // For example, "node" has a short form of "N".
      // If possible, use such a short form for the cache tag, as this helps
      // debugging caching issues.
      $short = $this->prefixMap[$first] ?? NULL;
      if ($short) {
        return "$short:$parts[1]";
      }
    }

    // Fallback to hashing the entire tag.
    return $this->hashTag($tag);
  }

}
