<?php

namespace Drupal\nuxt_multi_cache\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\nuxt_multi_cache\NuxtMultiCacheApi;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure settings for nuxt_multi_cache.
 */
class Settings extends ConfigFormBase {

  /**
   * Construct a new settings form.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\nuxt_multi_cache\NuxtMultiCacheApi $nuxtMultiCacheApi
   *   The Nuxt Multi Cache API service.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    protected NuxtMultiCacheApi $nuxtMultiCacheApi,
  ) {
    parent::__construct($configFactory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('config.factory'),
      $container->get('nuxt_multi_cache.api'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'nuxt_multi_cache_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Form constructor.
    $form = parent::buildForm($form, $form_state);

    // Default settings.
    $config = $this->config('nuxt_multi_cache.settings');

    $form['endpoint'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Endpoint URL'),
      '#default_value' => $config->get('endpoint'),
      '#description' => $this->t('URL of the nuxt-multi-cache endpoint.'),
    ];

    $form['token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Token'),
      '#default_value' => $config->get('token'),
      '#description' => $this->t('Auth token for the purge API.'),
    ];

    $form['frontend'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Frontend'),
      '#default_value' => $config->get('frontend'),
      '#description' => $this->t('URL of the frontend application. Used to generate route direct links.'),
    ];

    $ban_tags = $config->get('ban_tags') ?? [];
    $ban_tags = implode("\r\n", $ban_tags);
    $form['ban_tags'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Banned tags'),
      '#default_value' => $ban_tags,
      '#description' => $this->t('Enter cache tags (one per line) that should be filtered. These are not sent back in the cache tag header.'),
    ];

    $mapping = $config->get('mapping') ?? [];
    $mapping = implode("\r\n", $mapping);
    $form['mapping'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Tag mapping'),
      '#default_value' => $mapping,
      '#description' => $this->t(
        'Map common cache tags to reduce the HTTP header size. One mapping per line using the following syntax: tag_name.property.key;TPK'
      ),
    ];

    $form['purge_everything'] = [
      '#type' => 'submit',
      '#weight' => 100,
      '#value' => $this->t('Purge everything'),
      '#submit' => [[$this, 'purgeEverything']],
      '#attributes' => [
        'class' => ['button--danger'],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'nuxt_multi_cache.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('nuxt_multi_cache.settings');
    $config->set('endpoint', $form_state->getValue('endpoint'));
    $config->set('token', $form_state->getValue('token'));

    $mapping = explode("\r\n", $form_state->getValue('mapping'));
    $config->set('mapping', $mapping);

    $ban_tags = explode("\r\n", $form_state->getValue('ban_tags'));
    $config->set('ban_tags', $ban_tags);
    $config->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Purge everything.
   */
  public function purgeEverything() {
    if ($this->nuxtMultiCacheApi->purgeAll()) {
      $this->messenger()->addMessage('Purged everything successfully.');
    }
    else {
      $this->messenger()->addError('Failed to purge everything.');
    }
  }

}
