<?php

declare(strict_types=1);

namespace Drupal\nuxt_multi_cache;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\ClientException;

/**
 * Nuxt Multi Cache API Service.
 */
class NuxtMultiCacheApi {

  /**
   * The module config.
   */
  protected ImmutableConfig $config;

  /**
   * NuxtMultiCacheApi constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   Logger.
   * @param \GuzzleHttp\Client $client
   *   HTTP Client.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    protected LoggerChannelInterface $logger,
    protected ClientInterface $client,
    protected MessengerInterface $messenger,
    ConfigFactoryInterface $configFactory,
  ) {
    $this->config = $configFactory->get('nuxt_multi_cache.settings');
  }

  /**
   * Make a request to the Nuxt Multi Cache API.
   *
   * @param string $path
   *   The resource path.
   * @param array $parameters
   *   Query parameters.
   * @param string $method
   *   Query method, defaults to GET.
   *
   * @return array
   *   The response data.
   */
  public function request(string $path, array $parameters = [], string $method = 'get'): array {
    $url = $this->config->get('endpoint');
    $token = $this->config->get('token');

    if ($token) {
      $parameters = array_merge($parameters, [
        'headers' => [
          'x-nuxt-multi-cache-token' => $token,
        ],
      ]);
    }

    try {
      if ($method === 'get') {
        $response = $this->client->get($url . $path, $parameters);
      }
      else {
        $response = $this->client->post($url . $path, $parameters);
      }
      if (!empty($response->getBody())) {
        try {
          // Could be the case if the client does not throw exceptions on http
          // errors.
          return json_decode((string) $response->getBody(), TRUE, 512, JSON_THROW_ON_ERROR);
        }
        catch (\JsonException $exception) {
          $this->messenger->addError(
            'Unable to decode nuxt-multi-point-cache for body: @body and message: @message',
            [
              '@body' => $response->getBody(),
              '@message' => $exception->getMessage(),
            ]
                  );
          return [];
        }
      }
      else {
        $this->logger->error('The nuxt-multi-point-cache response body was empty.');
        return [];
      }
    }
    catch (ClientException $exception) {
      $code = $exception->getCode();
      $this->logger->error($exception->getMessage());
      if ($code === 403) {
        $this->messenger->addError('Invalid credentials provided for nuxt-multi-cache endpoint.');
      }
      return [];
    }

    return [];
  }

  /**
   * Get a list of cached pages.
   *
   * @return string[]
   *   The list of keys.
   */
  public function getPages(): array {
    $result = $this->request('/stats/route');

    return [
      'total' => 0,
      'rows' => $result['rows'] ?? [],
    ];
  }

  /**
   * Get the cached components.
   *
   * @param int $offset
   *   The offset.
   *
   * @return array
   *   The array of cached components.
   */
  public function getComponents($offset = 0): array {
    $result = $this->request('/stats/component', [
      'query' => [
        'offset' => $offset,
      ],
    ]);

    return [
      'total' => $result['total'] ?? 0,
      'rows' => $result['rows'] ?? [],
    ];
  }

  /**
   * Get the cached data items.
   *
   * @param int $offset
   *   The offset.
   *
   * @return array
   *   The cached components.
   */
  public function getData($offset = 0): array {
    $result = $this->request('/stats/data', [
      'query' => [
        'offset' => $offset,
      ],
    ]);

    return [
      'total' => $result['total'] ?? 0,
      'rows' => $result['rows'] ?? [],
    ];
  }

  /**
   * Purge pages by key.
   *
   * @param string[] $keys
   *   The keys of the pages to purge.
   *
   * @return bool
   *   If the operation was successful.
   */
  public function purgePages(array $keys): bool {
    $status = $this->request('/purge/route', ['json' => $keys], 'POST')['status'] ?? NULL;
    return $status === 'OK';
  }

  /**
   * Purge cache items by tag.
   *
   * @param string[] $tags
   *   The tags to purge.
   *
   * @return bool
   *   If the operation was successful.
   */
  public function purgeTags(array $tags): bool {
    $status = $this->request('/purge/tags', ['json' => $tags], 'POST')['status'] ?? NULL;
    return $status === 'OK';
  }

  /**
   * Purge components by keys.
   *
   * @param string[] $keys
   *   The keys to purge.
   *
   * @return bool
   *   If the operation was successful.
   */
  public function purgeComponents(array $keys): bool {
    $status = $this->request('/purge/component', ['json' => $keys], 'POST')['status'] ?? NULL;
    return $status === 'OK';
  }

  /**
   * Purge data items by key.
   *
   * @param string[] $keys
   *   The keys to purge.
   *
   * @return bool
   *   If the operation was successful.
   */
  public function purgeData(array $keys): bool {
    $status = $this->request('/purge/data', ['json' => $keys], 'POST')['status'];
    return $status === 'OK';
  }

  /**
   * Purge everything.
   *
   * @return bool
   *   If the operation was successful.
   */
  public function purgeAll(): bool {
    $status = $this->request('/purge/all', [], 'POST')['status'];
    return $status === 'OK';
  }

}
